using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.DirectoryServices.AccountManagement;
using System.IO;
using System.Management;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Security.AccessControl;
using System.Security.Principal;
using System.Text;
using System.Xml;
using log4net;

public static class ShareFolderManager
{
	public enum ShareMode : uint
	{
		SHARE_READ = 1179817u,
		SHARE_CHANGE = 1245631u,
		SHARE_FULL = 2032127u,
		SHARE_CUSTOM = 16777215u
	}

	public enum ErrorCode : uint
	{
		SUCCESS = 0u,
		ACCESS_DENIED = 2u,
		UNKNOWN_FAILURE = 8u,
		INVALID_NAME = 9u,
		INVALID_LEVEL = 10u,
		INVALID_PARAMETER = 21u,
		DUPLICATE_SHARE = 22u,
		REDIRECTED_PATH = 23u,
		UNKNOWN_DEVICE_DIRECTORY = 24u,
		NET_NAME_NOTFOUND = 25u,
		OTHER = 26u
	}

	public struct ShareUser
	{
		public ShareMode ShareMode { get; set; }

		public string AccountName { get; set; }

		public AceType AceType { get; set; }
	}

	public enum PrincipalType
	{
		User,
		Group
	}

	private enum SID_NAME_USE
	{
		SidTypeUser = 1,
		SidTypeGroup,
		SidTypeDomain,
		SidTypeAlias,
		SidTypeWellKnownGroup,
		SidTypeDeletedAccount,
		SidTypeInvalid,
		SidTypeUnknown,
		SidTypeComputer
	}

	private static readonly ILog LOG = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

	private static readonly string SHAREFOLDER_XPATH_PROPERTY = "/Objects/Object/Property";

	private static AceType aceType = AceType.AccessAllowed;

	public static ErrorCode shareFolderAdd(string path, string sharename, string description, params ShareUser[] shareUsers)
	{
		//IL_0005: Unknown result type (might be due to invalid IL or missing references)
		//IL_000a: Unknown result type (might be due to invalid IL or missing references)
		ManagementClass val = new ManagementClass("Win32_Share");
		ManagementBaseObject methodParameters = ((ManagementObject)val).GetMethodParameters("Create");
		methodParameters["Description"] = description;
		methodParameters["Name"] = sharename;
		methodParameters["Path"] = path;
		methodParameters["Type"] = 0;
		methodParameters["Access"] = GetSecurityDescriptor(shareUsers);
		return (ErrorCode)((ManagementObject)val).InvokeMethod("Create", methodParameters, (InvokeMethodOptions)null).Properties["ReturnValue"].Value;
	}

	public static void AddUsersAndPermissions(string DirectoryName, string UserAccount, FileSystemRights UserRights, AccessControlType AccessType)
	{
		DirectoryInfo directoryInfo = new DirectoryInfo(DirectoryName);
		DirectorySecurity accessControl = directoryInfo.GetAccessControl();
		string[] array = UserAccount.Split(new char[1] { '\\' });
		NTAccount identity = ((array.Length != 1) ? new NTAccount(array[0], array[1]) : new NTAccount(array[0]));
		accessControl.AddAccessRule(new FileSystemAccessRule(identity, UserRights, InheritanceFlags.ContainerInherit | InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessType));
		directoryInfo.SetAccessControl(accessControl);
	}

	public static void ChangeStatus(string shareName, params ShareUser[] shareUsers)
	{
		ChangeStatus(shareName, null, null, shareUsers);
	}

	private static void ChangeStatus(string shareName, int? shareUserCount, string description, params ShareUser[] shareUsers)
	{
		//IL_0023: Unknown result type (might be due to invalid IL or missing references)
		//IL_0062: Unknown result type (might be due to invalid IL or missing references)
		ManagementBaseObject descriptor = GetDescriptor(shareName);
		descriptor["DACL"] = GetAces(shareUsers);
		uint num = (uint)new ManagementObject($"\\root\\cimv2:Win32_Share.Name='{shareName}'").InvokeMethod("SetShareInfo", new object[3] { shareUserCount, description, descriptor });
		if (num != 0)
		{
			throw new ManagementException($"Error code : {num}.");
		}
	}

	public static ShareMode GetAccessRights(string shareName, string fullUserName)
	{
		//IL_0059: Unknown result type (might be due to invalid IL or missing references)
		//IL_0060: Expected O, but got Unknown
		LOG.DebugFormat("{0} start", (object)MethodBase.GetCurrentMethod().Name);
		uint num = 0u;
		ShareMode shareMode = ShareMode.SHARE_READ;
		ManagementBaseObject[] array = (ManagementBaseObject[])GetDescriptor(shareName).Properties["Dacl"].Value;
		foreach (ManagementBaseObject val in array)
		{
			try
			{
				ManagementBaseObject val2 = (ManagementBaseObject)val["Trustee"];
				int num2 = Convert.ToInt32(val["AceFlags"]);
				if (num2 != 19)
				{
					if (num < (uint)val["AccessMask"])
					{
						num = (uint)val["AccessMask"];
					}
					if ((uint)val["AceType"] == 0)
					{
						aceType = AceType.AccessAllowed;
					}
					else
					{
						aceType = AceType.AccessDenied;
					}
					string text = string.Format("{0}\\{1}", val2.Properties["Domain"].Value, val2.Properties["Name"].Value);
					LOG.DebugFormat("Trustee:{0}\\{1}  mask:0x{2:X}  AceFlags:0x{3:X}  type:0x{4:X}", new object[4]
					{
						text,
						(uint)val["AccessMask"],
						num2,
						aceType
					});
					if (string.Equals(text, fullUserName, StringComparison.CurrentCultureIgnoreCase))
					{
						break;
					}
				}
			}
			catch (Exception ex)
			{
				LOG.DebugFormat("{0}：Exception\n{1}", (object)MethodBase.GetCurrentMethod().Name, (object)ex.ToString());
				throw;
			}
		}
		if (num != 0 && Enum.IsDefined(typeof(ShareMode), num))
		{
			shareMode = (ShareMode)Enum.ToObject(typeof(ShareMode), num);
		}
		LOG.DebugFormat("{0} end. shareMode:{1}", (object)MethodBase.GetCurrentMethod().Name, (object)shareMode);
		return shareMode;
	}

	public static AceType GetAccessType()
	{
		return aceType;
	}

	[DllImport("advapi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
	private static extern bool LookupAccountSid(string lpSystemName, [MarshalAs(UnmanagedType.LPArray)] byte[] Sid, StringBuilder lpName, ref uint cchName, StringBuilder ReferencedDomainName, ref uint cchReferencedDomainName, out SID_NAME_USE peUse);

	public static bool TryGetPrincipalType(string domainQualifiedName, out PrincipalType type)
	{
		StringBuilder stringBuilder = new StringBuilder();
		uint cchName = (uint)stringBuilder.Capacity;
		StringBuilder stringBuilder2 = new StringBuilder();
		uint cchReferencedDomainName = (uint)stringBuilder2.Capacity;
		SecurityIdentifier securityIdentifier = new SecurityIdentifier(new NTAccount(domainQualifiedName).Translate(typeof(SecurityIdentifier)).Value);
		byte[] array = new byte[securityIdentifier.BinaryLength];
		securityIdentifier.GetBinaryForm(array, 0);
		if (LookupAccountSid(null, array, stringBuilder, ref cchName, stringBuilder2, ref cchReferencedDomainName, out var peUse))
		{
			switch (peUse)
			{
			case SID_NAME_USE.SidTypeGroup:
			case SID_NAME_USE.SidTypeAlias:
			case SID_NAME_USE.SidTypeWellKnownGroup:
				type = PrincipalType.Group;
				return true;
			case SID_NAME_USE.SidTypeUser:
				type = PrincipalType.User;
				return true;
			}
		}
		type = PrincipalType.User;
		return false;
	}

	public static List<string> GetUserList(string folderPath)
	{
		//IL_0038: Unknown result type (might be due to invalid IL or missing references)
		//IL_003f: Expected O, but got Unknown
		//IL_01d9: Expected O, but got Unknown
		//IL_00da: Unknown result type (might be due to invalid IL or missing references)
		//IL_00e4: Expected O, but got Unknown
		//IL_00df: Unknown result type (might be due to invalid IL or missing references)
		LOG.InfoFormat("{0}：start", (object)MethodBase.GetCurrentMethod().Name);
		List<string> list = new List<string>();
		DirectorySecurity accessControl = Directory.GetAccessControl(folderPath);
		string empty = string.Empty;
		string value = "AUTHORITY\\SYSTEM";
		string value2 = "Administrators";
		PrincipalContext val = new PrincipalContext((ContextType)0);
		foreach (FileSystemAccessRule accessRule in accessControl.GetAccessRules(includeExplicit: true, includeInherited: true, typeof(NTAccount)))
		{
			bool flag = false;
			if (accessRule.AccessControlType != 0)
			{
				continue;
			}
			try
			{
				empty = (accessRule.IdentityReference as NTAccount).Value;
			}
			catch (Exception ex)
			{
				LOG.ErrorFormat("{0}：User List Error", (object)MethodBase.GetCurrentMethod().Name, (object)ex.ToString());
				continue;
			}
			try
			{
				PrincipalType type;
				bool flag2 = TryGetPrincipalType(empty, out type);
				if (!flag2)
				{
					continue;
				}
				if (flag2 && type == PrincipalType.Group)
				{
					Principal val2 = new PrincipalSearcher((Principal)new GroupPrincipal(val, empty)).FindOne();
					if (val2 != null)
					{
						PrincipalCollection members = ((GroupPrincipal)((val2 is GroupPrincipal) ? val2 : null)).Members;
						try
						{
							foreach (Principal item in members)
							{
								try
								{
									string text = new SecurityIdentifier(item.Sid.ToString()).Translate(typeof(NTAccount)).ToString();
									if (!TryGetPrincipalType(text, out type) || type != 0)
									{
										continue;
									}
									for (int i = 0; i < list.Count; i++)
									{
										if (list[i] != null && list[i].Equals(text))
										{
											flag = true;
											break;
										}
									}
									if (!flag)
									{
										list.Add(text);
									}
								}
								catch (Exception ex2)
								{
									LOG.DebugFormat("{0}：Exception {1}", (object)MethodBase.GetCurrentMethod().Name, (object)ex2.ToString());
								}
							}
						}
						catch (PrincipalOperationException val3)
						{
							PrincipalOperationException val4 = val3;
							LOG.InfoFormat("{0}：PrincipalOperationException Error User in Folder. FolderName:{1},  {2}", (object)MethodBase.GetCurrentMethod().Name, (object)folderPath, (object)((object)val4).ToString());
						}
						catch (Exception ex3)
						{
							LOG.DebugFormat("{0}：Exception {1}", (object)MethodBase.GetCurrentMethod().Name, (object)ex3.ToString());
						}
						continue;
					}
					goto IL_0249;
				}
				goto IL_0249;
			}
			catch (Exception ex4)
			{
				LOG.DebugFormat("{0}：Exception {1}", (object)MethodBase.GetCurrentMethod().Name, (object)ex4.ToString());
			}
			continue;
			IL_0249:
			if (empty.IndexOf(value) > 0 || empty.IndexOf(value2) > 0)
			{
				continue;
			}
			for (int j = 0; j < list.Count; j++)
			{
				if (list[j] != null && list[j].Equals(empty))
				{
					flag = true;
					break;
				}
			}
			if (!flag)
			{
				list.Add(empty);
				Console.WriteLine(empty);
			}
		}
		LOG.InfoFormat("{0}：end", (object)MethodBase.GetCurrentMethod().Name);
		return list;
	}

	public static List<ShareUser> GetFolderCurrentUsers(string shareName)
	{
		LOG.DebugFormat("{0}：start", (object)MethodBase.GetCurrentMethod().Name);
		List<ShareUser> list = new List<ShareUser>();
		string text = executePowerShell($"ConvertTo-XML -as string -inputObject (Get-SmbShareAccess {shareName})");
		LOG.DebugFormat("Get-SmbShareAccess results:\n{0}", (object)text);
		XmlDocument xmlDocument = new XmlDocument();
		try
		{
			xmlDocument.LoadXml(text);
			XmlNodeList xmlNodeList = xmlDocument.SelectNodes(SHAREFOLDER_XPATH_PROPERTY);
			if (xmlNodeList.Count > 0 && string.Equals(xmlNodeList[0].Attributes["Type"].InnerText, "Microsoft.Management.Infrastructure.CimInstance"))
			{
				foreach (XmlElement item in xmlNodeList)
				{
					addAccessUser(list, item.ChildNodes);
				}
			}
			else
			{
				addAccessUser(list, xmlNodeList);
			}
		}
		catch (Exception ex)
		{
			LOG.ErrorFormat("{0}：共有フォルダー設定解析エラー\n{1}\n*******************************", (object)MethodBase.GetCurrentMethod().Name, (object)ex.ToString());
		}
		finally
		{
			LOG.DebugFormat("{0}：end", (object)MethodBase.GetCurrentMethod().Name);
		}
		return list;
	}

	private static void addAccessUser(List<ShareUser> accessUsers, XmlNodeList nodeList)
	{
		LOG.DebugFormat("{0} start", (object)MethodBase.GetCurrentMethod().Name);
		bool flag = false;
		ShareUser item = createShareUserObject(nodeList);
		if (string.IsNullOrEmpty(item.AccountName))
		{
			return;
		}
		foreach (ShareUser accessUser in accessUsers)
		{
			if (string.Equals(accessUser.AccountName, item.AccountName, StringComparison.CurrentCultureIgnoreCase) && object.Equals(accessUser.AceType, item.AceType))
			{
				flag = accessUser.ShareMode >= item.ShareMode;
				if (!flag)
				{
					accessUsers.Remove(accessUser);
				}
				break;
			}
		}
		if (!flag)
		{
			accessUsers.Add(item);
		}
		LOG.DebugFormat("{0} end. exist:{1}", (object)MethodBase.GetCurrentMethod().Name, (object)flag);
	}

	private static ShareUser createShareUserObject(XmlNodeList nodeList)
	{
		LOG.DebugFormat("{0} start", (object)MethodBase.GetCurrentMethod().Name);
		ShareUser result = default(ShareUser);
		AceType aceType = AceType.AccessAllowed;
		ShareMode shareMode = ShareMode.SHARE_READ;
		string shareName = string.Empty;
		string text = string.Empty;
		foreach (XmlNode node in nodeList)
		{
			switch (node.Attributes["Name"].Value)
			{
			case "AccessControlType":
			{
				string innerText = node.InnerText;
				if (!(innerText == "Allow"))
				{
					if (innerText == "Deny")
					{
						aceType = AceType.AccessDenied;
					}
				}
				else
				{
					aceType = AceType.AccessAllowed;
				}
				break;
			}
			case "AccessRight":
				switch (node.InnerText)
				{
				case "Change":
					shareMode = ShareMode.SHARE_CHANGE;
					break;
				case "Read":
					shareMode = ShareMode.SHARE_READ;
					break;
				case "Full":
					shareMode = ShareMode.SHARE_FULL;
					break;
				case "Custom":
					shareMode = ShareMode.SHARE_CUSTOM;
					break;
				}
				break;
			case "AccountName":
				text = node.InnerText;
				break;
			case "Name":
				shareName = node.InnerText;
				break;
			}
		}
		if (!string.IsNullOrEmpty(text))
		{
			if (shareMode == ShareMode.SHARE_CUSTOM)
			{
				shareMode = GetAccessRights(shareName, text);
			}
			result.AccountName = text;
			result.ShareMode = shareMode;
			result.AceType = aceType;
		}
		LOG.DebugFormat("{0} end. Existing share folder Account Name:{1} Access Right:{2} Access Type:{3}", new object[4]
		{
			MethodBase.GetCurrentMethod().Name,
			text,
			shareMode,
			aceType
		});
		return result;
	}

	private static byte[] CreateSidBinary(string accountName)
	{
		SecurityIdentifier obj = (SecurityIdentifier)new NTAccount(accountName).Translate(typeof(SecurityIdentifier));
		byte[] array = new byte[obj.BinaryLength];
		obj.GetBinaryForm(array, 0);
		return array;
	}

	private static ManagementBaseObject[] GetAces(ShareUser[] shareUsers)
	{
		ManagementBaseObject[] array = (ManagementBaseObject[])(object)new ManagementBaseObject[shareUsers.Length];
		for (int i = 0; i < shareUsers.Length; i++)
		{
			array[i] = (ManagementBaseObject)(object)GetAce(shareUsers[i]);
		}
		return array;
	}

	private static ManagementObject GetAce(ShareUser shareUser)
	{
		//IL_0005: Unknown result type (might be due to invalid IL or missing references)
		//IL_0010: Expected O, but got Unknown
		//IL_000b: Unknown result type (might be due to invalid IL or missing references)
		//IL_0011: Expected O, but got Unknown
		//IL_002d: Unknown result type (might be due to invalid IL or missing references)
		//IL_0038: Expected O, but got Unknown
		//IL_0033: Unknown result type (might be due to invalid IL or missing references)
		//IL_0038: Unknown result type (might be due to invalid IL or missing references)
		//IL_004f: Unknown result type (might be due to invalid IL or missing references)
		//IL_0060: Unknown result type (might be due to invalid IL or missing references)
		//IL_0077: Unknown result type (might be due to invalid IL or missing references)
		//IL_0084: Expected O, but got Unknown
		ManagementObject val = (ManagementObject)new ManagementClass(new ManagementPath("Win32_Trustee"), (ObjectGetOptions)null);
		((ManagementBaseObject)val)["SID"] = CreateSidBinary(shareUser.AccountName);
		return (ManagementObject)new ManagementClass(new ManagementPath("Win32_Ace"), (ObjectGetOptions)null)
		{
			["AccessMask"] = shareUser.ShareMode,
			["AceFlags"] = AceFlags.ObjectInherit | AceFlags.ContainerInherit,
			["AceType"] = shareUser.AceType,
			["Trustee"] = val
		};
	}

	private static ManagementBaseObject GetSecurityDescriptor(params ShareUser[] shareUsers)
	{
		//IL_0005: Unknown result type (might be due to invalid IL or missing references)
		//IL_0010: Expected O, but got Unknown
		//IL_000b: Unknown result type (might be due to invalid IL or missing references)
		//IL_0010: Unknown result type (might be due to invalid IL or missing references)
		//IL_0021: Unknown result type (might be due to invalid IL or missing references)
		//IL_0033: Expected O, but got Unknown
		return (ManagementBaseObject)new ManagementClass(new ManagementPath("Win32_SecurityDescriptor"), (ObjectGetOptions)null)
		{
			["ControlFlags"] = 4,
			["DACL"] = GetAces(shareUsers)
		};
	}

	private static ManagementBaseObject GetDescriptor(string shareName)
	{
		//IL_000b: Unknown result type (might be due to invalid IL or missing references)
		//IL_0064: Unknown result type (might be due to invalid IL or missing references)
		//IL_006a: Expected O, but got Unknown
		//IL_004a: Unknown result type (might be due to invalid IL or missing references)
		ManagementBaseObject obj = new ManagementObject($"\\root\\cimv2:Win32_LogicalShareSecuritySetting.Name='{shareName}'").InvokeMethod("GetSecurityDescriptor", (ManagementBaseObject)null, (InvokeMethodOptions)null);
		uint num = (uint)obj.Properties["ReturnValue"].Value;
		if (num != 0)
		{
			throw new ManagementException($"Error code : {num}.");
		}
		return (ManagementBaseObject)obj.Properties["Descriptor"].Value;
	}

	private static string executePowerShell(string command)
	{
		Process process = new Process();
		string result = string.Empty;
		try
		{
			process.StartInfo.FileName = "PowerShell.exe";
			process.StartInfo.UseShellExecute = false;
			process.StartInfo.RedirectStandardOutput = true;
			process.StartInfo.RedirectStandardInput = false;
			process.StartInfo.CreateNoWindow = true;
			process.StartInfo.Verb = "RunAs";
			process.StartInfo.Arguments = command;
			process.Start();
			result = process.StandardOutput.ReadToEnd();
		}
		catch (Exception ex)
		{
			LOG.ErrorFormat("{0}：Exception  :{1}", (object)MethodBase.GetCurrentMethod().Name, (object)ex.ToString());
		}
		finally
		{
			process.Close();
		}
		return result;
	}
}
